# Generated by Django 2.1.7 on 2019-04-25 09:22

import django.db.models.deletion
from django.db import migrations, models

import katka.fields


class Migration(migrations.Migration):

    dependencies = [
        ('katka', '0015_update_pipeline_statuses_fix'),
    ]

    operations = [
        migrations.CreateModel(
            name='ApplicationMetadata',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('created_username', katka.fields.AutoUsernameField(max_length=50)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('modified_username', katka.fields.AutoUsernameField(max_length=50)),
                ('deleted', models.BooleanField(default=False)),
                ('key', models.CharField(max_length=50)),
                ('value', models.CharField(max_length=200)),
                ('application', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='katka.Application')),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='applicationmetadata',
            unique_together={('application', 'key')},
        ),
    ]



# Generated by Django 2.1.7 on 2019-03-18 08:06

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('katka', '0010_rename_scmtype'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='credential',
            unique_together=set(),
        ),
        migrations.RemoveField(
            model_name='credential',
            name='slug',
        ),
    ]



# Generated by Django 2.1.5 on 2019-02-12 02:12

import django.db.models.deletion
from django.db import migrations, models

import encrypted_model_fields.fields
import katka.fields


class Migration(migrations.Migration):

    dependencies = [
        ('katka', '0003_credential_and_unique_slugs'),
    ]

    operations = [
        migrations.CreateModel(
            name='CredentialSecret',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('created_username', katka.fields.AutoUsernameField(max_length=50)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('modified_username', katka.fields.AutoUsernameField(max_length=50)),
                ('status', models.CharField(choices=[('active', 'active'), ('inactive', 'inactive')], default='active', max_length=50)),
                ('key', models.CharField(max_length=50)),
                ('value', encrypted_model_fields.fields.EncryptedCharField()),
                ('credential', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='katka.Credential')),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='credentialsecret',
            unique_together={('credential', 'key')},
        ),
    ]



from katka.models import Application, Credential, Project, SCMPipelineRun, SCMRepository, SCMService, Team
from rest_framework import serializers
from rest_framework.exceptions import NotFound, PermissionDenied
from rest_framework.relations import PrimaryKeyRelatedField


class GroupNameField(serializers.RelatedField):
    def to_internal_value(self, group_name):
        qs = self.get_queryset()
        try:
            return qs.get(name=group_name)
        except qs.model.DoesNotExist:
            raise NotFound

    def to_representation(self, group):
        return group.name


class PrimaryKeyRelated403Field(PrimaryKeyRelatedField):
    """
    Instead of replying with a 400 with a message 'does_not_exist', we would rather tell the user that they
    do not have permission because the referred team/credential/etc. either does not exist or they are not
    a member of a group that is linked to that object.
    """
    def fail(self, key, **kwargs):
        if key == 'does_not_exist':
            raise PermissionDenied(self.does_not_exist_message)

        return super().fail(key, **kwargs)


class TeamRelatedField(PrimaryKeyRelated403Field):
    does_not_exist_message = 'Team does not exist or you are not a member of its group'

    def get_queryset(self):
        """Only get the teams that are connected to a group that the user is a member of"""
        return Team.objects.filter(
            group__in=self.context['request'].user.groups.all(),
            deleted=False
        )


class ProjectRelatedField(PrimaryKeyRelated403Field):
    does_not_exist_message = 'Project does not exist or is not linked to your team'

    def get_queryset(self):
        """Only get the projects that are connected to a team that the user is a member of"""
        return Project.objects.filter(
            team__group__in=self.context['request'].user.groups.all(),
            team__deleted=False,
            deleted=False
        )


class CredentialRelatedField(PrimaryKeyRelated403Field):
    does_not_exist_message = 'Credential or team does not exist or is not linked to your group'

    def get_queryset(self):
        return Credential.objects.filter(
            team__group__in=self.context['request'].user.groups.all(),
            team__deleted=False,
            deleted=False,
        )


class SCMServiceRelatedField(PrimaryKeyRelated403Field):
    does_not_exist_message = 'SCMService does not exist'

    def get_queryset(self):
        return SCMService.objects.filter(
            deleted=False
        )


class SCMRepositoryRelatedField(PrimaryKeyRelated403Field):
    does_not_exist_message = 'SCMRepository does not exist'

    def get_queryset(self):
        return SCMRepository.objects.filter(
            scm_service__deleted=False,
            credential__deleted=False,
            deleted=False
        )


class ApplicationRelatedField(PrimaryKeyRelated403Field):
    does_not_exist_message = 'Application does not exist or does not belong to your team'

    def get_queryset(self):
        return Application.objects.filter(
            project__team__group__in=self.context['request'].user.groups.all(),
            project__team__deleted=False,
            project__deleted=False,
            deleted=False
        )


class SCMPipelineRunRelatedField(PrimaryKeyRelated403Field):
    does_not_exist_message = 'SCM Pipeline Run does not exist or does not belong to your team'

    def get_queryset(self):
        return SCMPipelineRun.objects.filter(
            application__project__team__group__in=self.context['request'].user.groups.all(),
            application__project__team__deleted=False,
            application__project__deleted=False,
            application__deleted=False,
            deleted=False
        )



from django.contrib import admin

from katka.fields import username_on_model
from katka.models import (
    Application, ApplicationMetadata, Credential, CredentialSecret, Project, SCMPipelineRun, SCMRelease, SCMRepository,
    SCMService, SCMStepRun, Team,
)


class WithUsernameAdminModel(admin.ModelAdmin):
    def save_model(self, request, obj, form, change):
        with username_on_model(self.model, request.user.username):
            super().save_model(request, obj, form, change)


@admin.register(Team)
class TeamAdmin(WithUsernameAdminModel):
    fields = ('name', 'group')


@admin.register(Project)
class ProjectAdmin(WithUsernameAdminModel):
    fields = ('name', 'slug', 'team')


@admin.register(Application)
class ApplicationAdmin(WithUsernameAdminModel):
    fields = ('name', 'slug', 'project', 'scm_repository')


@admin.register(Credential)
class CredentialAdmin(WithUsernameAdminModel):
    fields = ('name', 'credential_type', 'team')


@admin.register(CredentialSecret)
class CredentialSecretAdmin(WithUsernameAdminModel):
    fields = ('key', 'value', 'credential')


@admin.register(SCMService)
class SCMServiceAdmin(WithUsernameAdminModel):
    fields = ('scm_service_type', 'server_url')


@admin.register(SCMRepository)
class SCMRepositoryAdmin(WithUsernameAdminModel):
    fields = ('organisation', 'repository_name', 'credential', 'scm_service')
    list_display = ('pk', 'scm_service', 'organisation', 'repository_name')


@admin.register(SCMPipelineRun)
class SCMPipelineRunAdmin(WithUsernameAdminModel):
    fields = ('commit_hash', 'status', 'steps_total', 'steps_completed', 'pipeline_yaml', 'application')
    list_display = ('pk', 'application', 'commit_hash')
    list_filter = ('application__name',)


@admin.register(SCMStepRun)
class SCMStepRunAdmin(WithUsernameAdminModel):
    fields = ('slug', 'name', 'stage', 'status', 'output', 'scm_pipeline_run')
    list_display = ('pk', 'scm_pipeline_run', 'name', 'stage', 'status')
    list_filter = ('scm_pipeline_run__application__name',)


@admin.register(ApplicationMetadata)
class ApplicationMetadataAdmin(WithUsernameAdminModel):
    fields = ('key', 'value', 'application')
    list_display = ('pk', 'application', 'key')


@admin.register(SCMRelease)
class SCMReleaseAdmin(WithUsernameAdminModel):
    fields = ('name', 'status', 'started', 'ended', 'scm_pipeline_runs')
    list_display = ('pk', 'name')



# Generated by Django 2.2.6 on 2019-10-21 06:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('katka', '0020_multiple_pipeline_runs_per_release'),
    ]

    operations = [
        migrations.AddField(
            model_name='scmsteprun',
            name='tags',
            field=models.TextField(blank=True),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-21 07:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('katka', '0021_added_step_tags'),
    ]

    operations = [
        migrations.AlterField(
            model_name='scmrelease',
            name='status',
            field=models.CharField(choices=[('in progress', 'in progress'), ('failed', 'failed'), ('success', 'success')], default='in progress', max_length=30),
        ),
    ]



# Generated by Django 2.1.5 on 2019-05-06 04:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('katka', '0017_auto_20190501_0917'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='scmpipelinerun',
            options={'verbose_name': 'SCM pipeline', 'verbose_name_plural': 'SCM pipelines'},
        ),
        migrations.AlterModelOptions(
            name='scmrelease',
            options={'verbose_name': 'SCM release', 'verbose_name_plural': 'SCM releases'},
        ),
        migrations.AlterModelOptions(
            name='scmrepository',
            options={'verbose_name': 'SCM repository', 'verbose_name_plural': 'SCM repositories'},
        ),
        migrations.AlterModelOptions(
            name='scmservice',
            options={'verbose_name': 'SCM service', 'verbose_name_plural': 'SCM services'},
        ),
        migrations.AlterModelOptions(
            name='scmsteprun',
            options={'verbose_name': 'SCM step', 'verbose_name_plural': 'SCM steps'},
        ),
    ]



# Generated by Django 2.1.5 on 2019-02-19 03:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('katka', '0006_application_and_scm_repository'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='application',
            name='status',
        ),
        migrations.RemoveField(
            model_name='credential',
            name='status',
        ),
        migrations.RemoveField(
            model_name='credentialsecret',
            name='status',
        ),
        migrations.RemoveField(
            model_name='project',
            name='status',
        ),
        migrations.RemoveField(
            model_name='scmrepository',
            name='status',
        ),
        migrations.RemoveField(
            model_name='scmservice',
            name='status',
        ),
        migrations.RemoveField(
            model_name='team',
            name='status',
        ),
        migrations.AddField(
            model_name='application',
            name='deleted',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='credential',
            name='deleted',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='credentialsecret',
            name='deleted',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='project',
            name='deleted',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='scmrepository',
            name='deleted',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='scmservice',
            name='deleted',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='team',
            name='deleted',
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 2.1.5 on 2019-02-07 08:12

import uuid

import django.db.models.deletion
from django.db import migrations, models

import katka.fields


class Migration(migrations.Migration):

    dependencies = [
        ('katka', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Project',
            fields=[
                ('created', models.DateTimeField(auto_now_add=True)),
                ('created_username', katka.fields.AutoUsernameField(max_length=50)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('modified_username', katka.fields.AutoUsernameField(max_length=50)),
                ('status', models.CharField(choices=[('active', 'active'), ('inactive', 'inactive')], default='active', max_length=50)),
                ('public_identifier', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('slug', katka.fields.KatkaSlugField(max_length=10, unique=True)),
                ('name', models.CharField(max_length=100)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='team',
            name='slug',
            field=katka.fields.KatkaSlugField(default='NONE', max_length=10, unique=True),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='project',
            name='team',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='katka.Team'),
        ),
    ]



from contextlib import contextmanager

from django.db import models

from .exceptions import MissingUsername


@contextmanager
def username_on_model(model, username):
    """
    Create a context to set all usernames of all AutoUsernameFields of a Django model.

    Django's models are normally not aware of the request (by design),
    so automatically saving the username of the person who made the change
    to the object is not easy.

    Using a global object can lead to all kinds of subtle bugs, so we
    needed to find a way to pass the username in a different way; in this case
    explicit.

    Args:
        model: The Django model to check for instances of AutoUsernameFields
        username: The username to set in this context
    """

    # the following looks a bit hacky, but is actually the documented way of Django on how to get a list of fields:
    # https://docs.djangoproject.com/en/2.1/ref/models/meta/#retrieving-all-field-instances-of-a-model
    auto_username_fields = [field for field in model._meta.get_fields() if isinstance(field, AutoUsernameField)]

    for field in auto_username_fields:
        field.set_username(username)

    try:
        yield
    finally:
        for field in auto_username_fields:
            field.reset_username()


class AutoUsernameField(models.CharField):
    """
    Save the username on every modification

    To pass the username from a view, put all change operations within a context:

        def view(..., request, ...):
            with username_on_model(YourModel, request.username):
                code_that_gets_querylists_and_modifies_objects()

            return response

    For more information, see the 'username_on_model' context manager.
    """
    def __init__(self, *args, only_on_create=False, **kwargs):
        if 'max_length' not in kwargs:
            kwargs['max_length'] = 50

        self.only_on_create = only_on_create
        self._username = None
        self._prev_username = None

        super().__init__(*args, **kwargs)

    def set_username(self, username):
        self._prev_username, self._username = self._username, username

    def reset_username(self):
        self._username, self._prev_username = self._prev_username, None

    def pre_save(self, model_instance, add):
        if self._username is None:
            class_name = model_instance.__class__.__name__
            raise MissingUsername(
                f'No username set. Make sure the username is set with the "username_on_model({class_name}, username)" '
                'context manager'
            )

        if add or not self.only_on_create:
            setattr(model_instance, self.attname, self._username)

        return getattr(model_instance, self.attname)


class KatkaSlugField(models.SlugField):
    def __init__(self, *args, max_length=10, blank=False, null=False, **kwargs):
        super().__init__(*args, max_length=max_length, blank=blank, null=null, **kwargs)



import uuid

from django.contrib.auth.models import Group
from django.db import models

from encrypted_model_fields.fields import EncryptedCharField
from katka.auditedmodel import AuditedModel
from katka.constants import (
    PIPELINE_STATUS_CHOICES, PIPELINE_STATUS_INITIALIZING, RELEASE_STATUS_CHOICES, RELEASE_STATUS_IN_PROGRESS,
    STEP_STATUS_CHOICES, STEP_STATUS_NOT_STARTED,
)
from katka.fields import KatkaSlugField


class Team(AuditedModel):
    public_identifier = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    slug = KatkaSlugField(unique=True)
    name = models.CharField(max_length=100)
    group = models.ForeignKey(Group, on_delete=models.CASCADE)

    def __str__(self):  # pragma: no cover
        return f'{self.group.name}'


class Project(AuditedModel):
    public_identifier = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    slug = KatkaSlugField()
    name = models.CharField(max_length=100)
    team = models.ForeignKey(Team, on_delete=models.CASCADE)

    class Meta:
        unique_together = ('team', 'slug')

    def __str__(self):  # pragma: no cover
        return f'{self.name}'


class Credential(AuditedModel):
    public_identifier = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=100)
    credential_type = models.CharField(max_length=50)
    team = models.ForeignKey(Team, on_delete=models.CASCADE)

    def __str__(self):  # pragma: no cover
        return f'{self.credential_type}/{self.name}'


class CredentialSecret(AuditedModel):
    key = models.CharField(max_length=50)
    value = EncryptedCharField(max_length=200)
    credential = models.ForeignKey(Credential, on_delete=models.CASCADE)

    class Meta:
        unique_together = ('credential', 'key')

    def __str__(self):  # pragma: no cover
        return f'{self.credential.name}/{self.key}'


class SCMService(AuditedModel):
    class Meta:
        verbose_name = 'SCM service'
        verbose_name_plural = 'SCM services'

    public_identifier = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    scm_service_type = models.CharField(max_length=48)
    server_url = models.URLField(null=False)

    def __str__(self):  # pragma: no cover
        return f'{self.scm_service_type}/{self.server_url}'


class SCMRepository(AuditedModel):
    class Meta:
        verbose_name = 'SCM repository'
        verbose_name_plural = 'SCM repositories'

    public_identifier = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    organisation = models.CharField(max_length=128)
    repository_name = models.CharField(max_length=128)
    credential = models.ForeignKey(Credential, on_delete=models.PROTECT)
    scm_service = models.ForeignKey(SCMService, on_delete=models.PROTECT)

    def __str__(self):  # pragma: no cover
        return f'{self.organisation}/{self.repository_name}'


class Application(AuditedModel):
    public_identifier = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    slug = KatkaSlugField()
    name = models.CharField(max_length=100)
    project = models.ForeignKey(Project, on_delete=models.PROTECT)
    scm_repository = models.OneToOneField(SCMRepository, on_delete=models.PROTECT)

    class Meta:
        unique_together = ('project', 'slug')

    def __str__(self):  # pragma: no cover
        return f'{self.name}'


# Pipeline run results
class SCMPipelineRun(AuditedModel):
    class Meta:
        verbose_name = 'SCM pipeline'
        verbose_name_plural = 'SCM pipelines'

    public_identifier = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    commit_hash = models.CharField(max_length=64)  # A SHA-1 hash is 40 characters, SHA-256 is 64 characters
    status = models.CharField(max_length=30, choices=PIPELINE_STATUS_CHOICES, default=PIPELINE_STATUS_INITIALIZING)
    steps_total = models.PositiveSmallIntegerField(default=0)
    steps_completed = models.PositiveSmallIntegerField(default=0)
    pipeline_yaml = models.TextField(default='---')
    application = models.ForeignKey(Application, on_delete=models.PROTECT)


class SCMStepRun(AuditedModel):
    class Meta:
        verbose_name = 'SCM step'
        verbose_name_plural = 'SCM steps'

    public_identifier = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    slug = KatkaSlugField(max_length=30)
    name = models.CharField(max_length=100)
    stage = models.CharField(max_length=100)
    status = models.CharField(max_length=30, choices=STEP_STATUS_CHOICES, default=STEP_STATUS_NOT_STARTED)
    output = models.TextField(blank=True)
    sequence_id = models.CharField(max_length=30, blank=True, null=True)
    # The format of a sequence ID is: <stage_nr>.<step_nr>-<parallel_nr>, with the following explanation:
    #
    #   stage_nr: the sequence number of the stage. The first stage gets number 1, the next 2, then 3, etc.
    #   step_nr: the sequence number of the step inside a stage. Each first step of the stage gets 1, then 2,
    #            then 3, etc.
    #   parallel_nr: these steps can be done in parallel so they only need a sequence so that the interface will show
    #                the steps in a consistent way.
    #
    # an example would be: "1.1-1" or "1.1" if there are no parallel steps. There should be zero padding, so if
    # there are more than 9 stages, it should be "01.1", or if there are more than 9 steps: "1.01".
    # This allows easy sorting for e.g. frontends.
    scm_pipeline_run = models.ForeignKey(SCMPipelineRun, on_delete=models.PROTECT)
    tags = models.TextField(blank=True)


# SCM Releases, comprises a range of commits that are released
class SCMRelease(AuditedModel):
    class Meta:
        verbose_name = 'SCM release'
        verbose_name_plural = 'SCM releases'

    public_identifier = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=255)
    status = models.CharField(max_length=30, choices=RELEASE_STATUS_CHOICES, default=RELEASE_STATUS_IN_PROGRESS)
    started = models.DateTimeField(null=True)
    ended = models.DateTimeField(null=True)
    scm_pipeline_runs = models.ManyToManyField(SCMPipelineRun)


class ApplicationMetadata(AuditedModel):
    key = models.CharField(max_length=50)
    value = models.CharField(max_length=200)
    application = models.ForeignKey(Application, on_delete=models.CASCADE)

    class Meta:
        unique_together = ('application', 'key')

    def __str__(self):  # pragma: no cover
        return f'{self.application.name}/{self.key}'
